/* 
  CESYS Software License - Version 1.0 - January 1st, 2016

  Permission is hereby granted, free of charge, to any person or organization
  obtaining a copy of the software and accompanying documentation covered by
  this license (the "Software") to use, reproduce, display, distribute,
  execute, and transmit the Software, and to prepare derivative works of the
  Software, and to permit third-parties to whom the Software is furnished to
  do so, all subject to the following:

  The software must only be used to operate hardware manufactured by 
  CESYS GmbH, Herzogenaurach.

  The copyright notices in the Software and this entire statement, including
  the above license grant, this restriction and the following disclaimer,
  must be included in all copies of the Software, in whole or in part, and
  all derivative works of the Software, unless such copies or derivative
  works are solely in the form of machine-executable object code generated by
  a source language processor.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
  SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
  FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
  
  Written by Thomas Hoppe <thomas.hoppe@cesys.com>, 2016
*/  

package com.cesys.cebo.cebomsr;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Offers functionality for analog outputs. Direct construction
 * is not possible. Instances can be retrieved calling getAnalogOutputs()
 * of the respective {@link Device} class.
 *
 * @author Thomas Hoppe
 */
public final class AnalogOutput implements Output {
	private final List<Range> _supportedRanges = new ArrayList<Range>();
	private final Device _device;
	private final int _interfaceId;
	private final int _index;
	private final String _name;

	int getInterfaceId() {
		return _interfaceId;
	}

	int getIndex() {
		return _index;
	}

	AnalogOutput(Device device, int interfaceId, int index) throws IOException {
		_device = device;
		_interfaceId = interfaceId;
		_index = index;

		int[] rangeId = new int[1];
		for (int i = 0;; ++i) {
			SystemConnector.checkError(SystemConnector.CeboMsrEnumerateRanges(_device.getHandle(), getInterfaceId(), i, rangeId));
			if (rangeId[0] == SystemConnector.CeboMsrRangeInvalid)
				break;

			float[] minValue = new float[1], maxValue = new float[1];
			SystemConnector.checkError(SystemConnector.CeboMsrGetRangeMinMax(device.getHandle(), getInterfaceId(), rangeId[0], minValue, maxValue));
			_supportedRanges.add(new Range(rangeId[0], minValue[0], maxValue[0], 0));
		}
		byte[] tmp = new byte[SystemConnector.MaxIdentLength];
		SystemConnector.checkError(SystemConnector.CeboMsrGetPeripheralName(device.getHandle(), interfaceId, tmp));
		_name = Utils.convertByteArrayToString(tmp);
	}

	/**
	 * Return list of the supported ranges.
	 */
	public List<Range> getSupportedRanges() {
		return Collections.unmodifiableList(_supportedRanges);
	}

	/**
	 * Set new range.
	 */
	public void setParameters(Range range) throws IOException {
		// No device currently supports this, included for future functionality.
//		int r[] = new int[1];
//		SystemConnector.checkError(SystemConnector.CeboMsrSetAnalogInputParameters(_device.getHandle(), getInterfaceId(), range.getInterfaceId(), -1, r));
	}

	/**
	 * Read active range level.
	 */
	public Range getRange() throws IOException {
		// No device currently supports this, included for future functionality.
		return _supportedRanges.get(0);
	}

	/**
	 * Set voltage level directly.
	 * @param value Value in volt to set.
	 */
	public void write(float value) throws IOException {
		SystemConnector.checkError(SystemConnector.CeboMsrWriteAnalogOutput(_device.getHandle(), getInterfaceId(), value));
	}

	/**
	 * Returns name of the component.
	 */
	public String getName() {
		return _name;
	}

	public String toString() {
		return "AnalogOutput" + _index;
	}
}
